@@ -14,8 +14,14 @@
 
 import copy
 
+from openstack.network.v2 import agent as sdk_agent
+from openstack.network.v2 import floating_ip as sdk_fip
 from openstack.network.v2 import network as sdk_net
+from openstack.network.v2 import network_ip_availability as \
+    sdk_ip_availability
 from openstack.network.v2 import port as sdk_port
+from openstack.network.v2 import port_forwarding as sdk_pf
+from openstack.network.v2 import router as sdk_router
 from openstack.network.v2 import subnet as sdk_subnet
 from openstack.network.v2 import subnet_pool as sdk_subnet_pool
 from openstack.network.v2 import trunk as sdk_trunk
@@ -60,16 +66,6 @@ def data(TEST):
     TEST.neutron_availability_zones = utils.TestDataContainer()
 
     # Data return by neutronclient.
-    TEST.api_agents = utils.TestDataContainer()
-    TEST.api_networks = utils.TestDataContainer()
-    TEST.api_subnets = utils.TestDataContainer()
-    TEST.api_subnetpools = utils.TestDataContainer()
-    TEST.api_ports = utils.TestDataContainer()
-    TEST.api_trunks = utils.TestDataContainer()
-    TEST.api_routers = utils.TestDataContainer()
-    TEST.api_routers_with_routes = utils.TestDataContainer()
-    TEST.api_floating_ips = utils.TestDataContainer()
-    TEST.api_port_forwardings = utils.TestDataContainer()
     TEST.api_security_groups = utils.TestDataContainer()
     TEST.api_security_group_rules = utils.TestDataContainer()
     TEST.api_pools = utils.TestDataContainer()
@@ -77,15 +73,12 @@ def data(TEST):
     TEST.api_members = utils.TestDataContainer()
     TEST.api_monitors = utils.TestDataContainer()
     TEST.api_extensions = utils.TestDataContainer()
-    TEST.api_ip_availability = utils.TestDataContainer()
     TEST.api_rbac_policies = utils.TestDataContainer()
     TEST.api_qos_policies = utils.TestDataContainer()
     TEST.api_dscp_mark_rule = utils.TestDataContainer()
     TEST.api_bandwidth_limit_rule = utils.TestDataContainer()
     TEST.api_minimum_bandwidth_rule = utils.TestDataContainer()
     TEST.api_minimum_packet_rate_rule = utils.TestDataContainer()
-    TEST.api_tp_trunks = utils.TestDataContainer()
-    TEST.api_tp_ports = utils.TestDataContainer()
 
     # Data returned by SDK:
     TEST.api_networks_sdk = list()
@@ -95,6 +88,12 @@ def data(TEST):
     TEST.api_ports_sdk = list()
     TEST.api_tp_ports_sdk = list()
     TEST.api_subnetpools_sdk = list()
+    TEST.api_routers_sdk = list()
+    TEST.api_routers_with_routes_sdk = list()
+    TEST.api_agents_sdk = list()
+    TEST.api_ip_availability_sdk = list()
+    TEST.api_port_forwardings_sdk = list()
+    TEST.api_floating_ips_sdk = list()
 
     # 1st network.
     network_dict = {'is_admin_state_up': True,
@@ -139,12 +138,9 @@ def data(TEST):
         'ipv6_address_mode': 'slaac'
     }
 
-    TEST.api_networks.add(network_dict)
     TEST.api_networks_sdk.append(sdk_net.Network(**network_dict))
     TEST.api_subnets_sdk.append(sdk_subnet.Subnet(**subnet_dict))
     TEST.api_subnets_sdk.append(sdk_subnet.Subnet(**subnetv6_dict))
-    TEST.api_subnets.add(subnet_dict)
-    TEST.api_subnets.add(subnetv6_dict)
 
     network = copy.deepcopy(network_dict)
     subnet = neutron.Subnet(subnet_dict)
@@ -177,7 +173,6 @@ def data(TEST):
         'security_group_ids': [],
     }
 
-    TEST.api_ports.add(port_dict)
     TEST.api_ports_sdk.append(sdk_port.Port(**port_dict))
     TEST.ports.add(neutron.Port(port_dict))
 
@@ -205,7 +200,6 @@ def data(TEST):
             '27a5c9a1-bdbb-48ac-833a-2e4b5f54b31d'
         ],
     }
-    TEST.api_ports.add(port_dict)
     TEST.api_ports_sdk.append(sdk_port.Port(**port_dict))
     TEST.ports.add(neutron.Port(port_dict))
 
@@ -227,7 +221,6 @@ def data(TEST):
         'binding:host_id': 'host',
         'security_group_ids': [],
     }
-    TEST.api_ports.add(port_dict)
     TEST.api_ports_sdk.append(sdk_port.Port(**port_dict))
     TEST.ports.add(neutron.Port(port_dict))
     port_dict = {
@@ -246,7 +239,6 @@ def data(TEST):
         'binding:host_id': 'host',
         'security_group_ids': [],
     }
-    TEST.api_ports.add(port_dict)
     TEST.api_ports_sdk.append(sdk_port.Port(**port_dict))
     TEST.ports.add(neutron.Port(port_dict))
 
@@ -267,7 +259,6 @@ def data(TEST):
         'binding:host_id': '',
         'security_group_ids': [],
     }
-    TEST.api_ports.add(port_dict)
     TEST.api_ports_sdk.append(sdk_port.Port(**port_dict))
     TEST.ports.add(neutron.Port(port_dict))
 
@@ -300,9 +291,7 @@ def data(TEST):
                    'tenant_id': network_dict['tenant_id']}
 
     TEST.api_networks_sdk.append(sdk_net.Network(**network_dict))
-    TEST.api_networks.add(network_dict)
     TEST.api_subnets_sdk.append(sdk_subnet.Subnet(**subnet_dict))
-    TEST.api_subnets.add(subnet_dict)
 
     network = copy.deepcopy(network_dict)
     subnet = neutron.Subnet(subnet_dict)
@@ -330,7 +319,6 @@ def data(TEST):
         ],
     }
 
-    TEST.api_ports.add(port_dict)
     TEST.api_ports_sdk.append(sdk_port.Port(**port_dict))
     TEST.ports.add(neutron.Port(port_dict))
 
@@ -361,9 +349,7 @@ def data(TEST):
     ext_net = network_dict
 
     TEST.api_networks_sdk.append(sdk_net.Network(**network_dict))
-    TEST.api_networks.add(network_dict)
     TEST.api_subnets_sdk.append(sdk_subnet.Subnet(**subnet_dict))
-    TEST.api_subnets.add(subnet_dict)
 
     network = copy.deepcopy(network_dict)
     subnet = neutron.Subnet(subnet_dict)
@@ -398,9 +384,7 @@ def data(TEST):
                    'tenant_id': network_dict['tenant_id']}
 
     TEST.api_networks_sdk.append(sdk_net.Network(**network_dict))
-    TEST.api_networks.add(network_dict)
     TEST.api_subnets_sdk.append(sdk_subnet.Subnet(**subnet_dict))
-    TEST.api_subnets.add(subnet_dict)
 
     network = copy.deepcopy(network_dict)
     subnet = neutron.Subnet(subnet_dict)
@@ -434,9 +418,7 @@ def data(TEST):
                    'tenant_id': network_dict['tenant_id']}
 
     TEST.api_networks_sdk.append(sdk_net.Network(**network_dict))
-    TEST.api_networks.add(network_dict)
     TEST.api_subnets_sdk.append(sdk_subnet.Subnet(**subnet_dict))
-    TEST.api_subnets.add(subnet_dict)
 
     network = copy.deepcopy(network_dict)
     subnet = neutron.Subnet(subnet_dict)
@@ -470,9 +452,7 @@ def data(TEST):
                    'tenant_id': network_dict['tenant_id']}
 
     TEST.api_networks_sdk.append(sdk_net.Network(**network_dict))
-    TEST.api_networks.add(network_dict)
     TEST.api_subnets_sdk.append(sdk_subnet.Subnet(**subnet_dict))
-    TEST.api_subnets.add(subnet_dict)
 
     network = copy.deepcopy(network_dict)
     subnet = neutron.Subnet(subnet_dict)
@@ -506,9 +486,7 @@ def data(TEST):
                    'tenant_id': network_dict['tenant_id']}
 
     TEST.api_networks_sdk.append(sdk_net.Network(**network_dict))
-    TEST.api_networks.add(network_dict)
     TEST.api_subnets_sdk.append(sdk_subnet.Subnet(**subnet_dict))
-    TEST.api_subnets.add(subnet_dict)
 
     network = copy.deepcopy(network_dict)
     subnet = neutron.Subnet(subnet_dict)
@@ -542,9 +520,7 @@ def data(TEST):
                    'tenant_id': network_dict['tenant_id']}
 
     TEST.api_networks_sdk.append(sdk_net.Network(**network_dict))
-    TEST.api_networks.add(network_dict)
     TEST.api_subnets_sdk.append(sdk_subnet.Subnet(**subnet_dict))
-    TEST.api_subnets.add(subnet_dict)
 
     network = copy.deepcopy(network_dict)
     subnet = neutron.Subnet(subnet_dict)
@@ -579,9 +555,7 @@ def data(TEST):
                    'ipv6_modes': 'none/none'}
 
     TEST.api_networks_sdk.append(sdk_net.Network(**network_dict))
-    TEST.api_networks.add(network_dict)
     TEST.api_subnets_sdk.append(sdk_subnet.Subnet(**subnet_dict))
-    TEST.api_subnets.add(subnet_dict)
 
     network = copy.deepcopy(network_dict)
     subnet = neutron.Subnet(subnet_dict)
@@ -616,9 +590,7 @@ def data(TEST):
                    'ipv6_modes': 'slaac/slaac'}
 
     TEST.api_networks_sdk.append(sdk_net.Network(**network_dict))
-    TEST.api_networks.add(network_dict)
     TEST.api_subnets_sdk.append(sdk_subnet.Subnet(**subnet_dict))
-    TEST.api_subnets.add(subnet_dict)
 
     network = copy.deepcopy(network_dict)
     subnet = neutron.Subnet(subnet_dict)
@@ -643,7 +615,6 @@ def data(TEST):
         'binding:host_id': 'host',
         'security_group_ids': [],
     }
-    TEST.api_ports.add(port_dict)
     TEST.api_ports_sdk.append(sdk_port.Port(**port_dict))
     TEST.ports.add(neutron.Port(port_dict))
 
@@ -657,7 +628,6 @@ def data(TEST):
                   'port_id': '895d375c-1447-11e7-a52f-f7f280bbc809',
                   'id': '94fcb9e8-1447-11e7-bed6-8b8c4ac74491'}
 
-    TEST.api_trunks.add(trunk_dict)
     TEST.trunks.add(neutron.Trunk(trunk_dict))
     TEST.api_trunks_sdk.append(sdk_trunk.Trunk(**trunk_dict))
 
@@ -665,27 +635,33 @@ def data(TEST):
                    'name': 'router1',
                    'status': 'ACTIVE',
                    'admin_state_up': True,
+                   'is_admin_state_up': True,
                    'distributed': True,
+                   'is_distributed': True,
                    'external_gateway_info':
                        {'network_id': ext_net['id']},
                    'tenant_id': '1',
                    'availability_zone_hints': ['nova']}
-    TEST.api_routers.add(router_dict)
+    TEST.api_routers_sdk.append(sdk_router.Router(**router_dict))
     TEST.routers.add(neutron.Router(router_dict))
     router_dict = {'id': '10e3dc42-1ce1-4d48-87cf-7fc333055d6c',
                    'name': 'router2',
                    'status': 'ACTIVE',
                    'admin_state_up': False,
+                   'is_admin_state_up': False,
                    'distributed': False,
+                   'is_distributed': False,
                    'external_gateway_info': None,
                    'tenant_id': '1'}
-    TEST.api_routers.add(router_dict)
+    TEST.api_routers_sdk.append(sdk_router.Router(**router_dict))
     TEST.routers.add(neutron.Router(router_dict))
     router_dict = {'id': '7180cede-bcd8-4334-b19f-f7ef2f331f53',
                    'name': 'rulerouter',
                    'status': 'ACTIVE',
                    'admin_state_up': True,
+                   'is_admin_state_up': True,
                    'distributed': False,
+                   'is_distributed': False,
                    'external_gateway_info':
                        {'network_id': ext_net['id']},
                    'tenant_id': '1',
@@ -699,13 +675,15 @@ def data(TEST):
                                      'source': 'any',
                                      'destination': '8.8.8.8/32',
                                      'nexthops': ['1.0.0.2', '1.0.0.1']}]}
-    TEST.api_routers.add(router_dict)
+    TEST.api_routers_sdk.append(sdk_router.Router(**router_dict))
     TEST.routers_with_rules.add(neutron.Router(router_dict))
     router_dict_with_route = {'id': '725c24c9-061b-416b-b9d4-012392b32fd9',
                               'name': 'routerouter',
                               'status': 'ACTIVE',
                               'admin_state_up': True,
+                              'is_admin_state_up': True,
                               'distributed': False,
+                              'is_distributed': False,
                               'external_gateway_info':
                                   {'network_id': ext_net['id']},
                               'tenant_id': '1',
@@ -713,7 +691,8 @@ def data(TEST):
                                           'destination': '172.0.0.0/24'},
                                          {'nexthop': '10.0.0.2',
                                           'destination': '172.1.0.0/24'}]}
-    TEST.api_routers_with_routes.add(router_dict_with_route)
+    TEST.api_routers_with_routes_sdk.append(
+        sdk_router.Router(**router_dict_with_route))
     TEST.routers_with_routes.add(neutron.Router(router_dict_with_route))
 
     # Floating IP.
@@ -726,7 +705,7 @@ def data(TEST):
                 'port_id': None,
                 'port_forwardings': [],
                 'router_id': None}
-    TEST.api_floating_ips.add(fip_dict)
+    TEST.api_floating_ips_sdk.append(sdk_fip.FloatingIP(**fip_dict))
     fip_with_instance = copy.deepcopy(fip_dict)
     fip_with_instance.update({'instance_id': None,
                               'instance_type': None})
@@ -741,7 +720,7 @@ def data(TEST):
                 'fixed_ip_address': assoc_port['fixed_ips'][0]['ip_address'],
                 'port_id': assoc_port['id'],
                 'router_id': router_dict['id']}
-    TEST.api_floating_ips.add(fip_dict)
+    TEST.api_floating_ips_sdk.append(sdk_fip.FloatingIP(**fip_dict))
     fip_with_instance = copy.deepcopy(fip_dict)
     fip_with_instance.update({'instance_id': '1',
                               'instance_type': 'compute'})
@@ -749,18 +728,23 @@ def data(TEST):
 
     # port forwardings
 
-    TEST.api_port_forwardings.add({
+    pf_dict = {
+        "name": "pf0",
         "protocol": "tcp",
         "internal_ip_address": "10.0.0.11",
         "internal_port": 25,
         "internal_port_id": "1238be08-a2a8-4b8d-addf-fb5e2250e480",
         "external_port": 2230,
-        "internal_port_range": "25:25",
-        "external_port_range": "2230:2230",
+        "internal_port_range": "25:26",
+        "external_port_range": "2230:2231",
         "description": "",
-        "id": "e0a0274e-4d19-4eab-9e12-9e77a8caf3ea"
-    })
-    TEST.api_port_forwardings.add({
+        "id": "e0a0274e-4d19-4eab-9e12-9e77a8caf3ea",
+        'floatingip_id': fip_dict['id'],
+        "location": None
+    }
+    TEST.api_port_forwardings_sdk.append(sdk_pf.PortForwarding(**pf_dict))
+    pf_dict = {
+        "name": "pf1",
         "protocol": "tcp",
         "internal_port": 80,
         "external_port": 8080,
@@ -769,22 +753,29 @@ def data(TEST):
         "internal_port_id": "2057ec54-8be2-11eb-8dcd-0242ac130003",
         "external_port_range": "8080:8090",
         "description": "using port ranges",
-        "id": "0f23a90a-8be2-11eb-8dcd-0242ac130003"
-    })
-    TEST.api_port_forwardings.add({
+        "id": "0f23a90a-8be2-11eb-8dcd-0242ac130003",
+        'floatingip_id': fip_dict['id'],
+        "location": None
+    }
+    TEST.api_port_forwardings_sdk.append(sdk_pf.PortForwarding(**pf_dict))
+    pf_dict = {
+        "name": "pf2",
         "protocol": "tcp",
         "internal_ip_address": "10.0.0.24",
         "internal_port": 25,
         "internal_port_id": "070ef0b2-0175-4299-be5c-01fea8cca522",
         "external_port": 2229,
-        "internal_port_range": "25:25",
-        "external_port_range": "2229:2229",
+        "internal_port_range": "25:27",
+        "external_port_range": "2229:2231",
         "description": "Some description",
-        "id": "1798dc82-c0ed-4b79-b12d-4c3c18f90eb2"
-    })
+        "id": "1798dc82-c0ed-4b79-b12d-4c3c18f90eb2",
+        'floatingip_id': fip_dict['id'],
+        "location": None
+    }
+    TEST.api_port_forwardings_sdk.append(sdk_pf.PortForwarding(**pf_dict))
 
     TEST.port_forwardings.add(neutron.PortForwarding(
-        TEST.api_port_forwardings.first(), fip_dict['id']
+        TEST.api_port_forwardings_sdk[0].to_dict(), fip_dict['id']
     ))
 
     # Security group.
@@ -929,7 +920,6 @@ def data(TEST):
                        'shared': False,
                        'tenant_id': '1'}
 
-    TEST.api_subnetpools.add(subnetpool_dict)
     TEST.api_subnetpools_sdk.append(
         (sdk_subnet_pool.SubnetPool(**subnetpool_dict)))
     subnetpool = neutron.SubnetPool(subnetpool_dict)
@@ -947,7 +937,6 @@ def data(TEST):
                        'shared': False,
                        'tenant_id': '1'}
 
-    TEST.api_subnetpools.add(subnetpool_dict)
     TEST.api_subnetpools_sdk.append(
         (sdk_subnet_pool.SubnetPool(**subnetpool_dict)))
     subnetpool = neutron.SubnetPool(subnetpool_dict)
@@ -1016,9 +1005,9 @@ def data(TEST):
     # 1st agent.
     agent_dict = {"binary": "neutron-openvswitch-agent",
                   "description": None,
-                  "admin_state_up": True,
-                  "heartbeat_timestamp": "2013-07-26 06:51:47",
-                  "alive": True,
+                  "is_admin_state_up": True,
+                  "last_heartbeat_at": "2013-07-26 06:51:47",
+                  "is_alive": True,
                   "id": "c876ff05-f440-443e-808c-1d34cda3e88a",
                   "topic": "N/A",
                   "host": "devstack001",
@@ -1026,15 +1015,15 @@ def data(TEST):
                   "started_at": "2013-07-26 05:23:28",
                   "created_at": "2013-07-26 05:23:28",
                   "configurations": {"devices": 2}}
-    TEST.api_agents.add(agent_dict)
+    TEST.api_agents_sdk.append(sdk_agent.Agent(**agent_dict))
     TEST.agents.add(neutron.Agent(agent_dict))
 
     # 2nd agent.
     agent_dict = {"binary": "neutron-dhcp-agent",
                   "description": None,
-                  "admin_state_up": True,
-                  "heartbeat_timestamp": "2013-07-26 06:51:48",
-                  "alive": True,
+                  "is_admin_state_up": True,
+                  "last_heartbeat_at": "2013-07-26 06:51:48",
+                  "is_alive": True,
                   "id": "f0d12e3d-1973-41a2-b977-b95693f9a8aa",
                   "topic": "dhcp_agent",
                   "host": "devstack001",
@@ -1048,7 +1037,7 @@ def data(TEST):
                       "dhcp_driver": "neutron.agent.linux.dhcp.Dnsmasq",
                       "networks": 1,
                       "ports": 1}}
-    TEST.api_agents.add(agent_dict)
+    TEST.api_agents_sdk.append(sdk_agent.Agent(**agent_dict))
     TEST.agents.add(neutron.Agent(agent_dict))
 
     # Service providers.
@@ -1075,7 +1064,6 @@ def data(TEST):
         'binding:host_id': 'host',
         'security_group_ids': [],
     }
-    TEST.api_ports.add(port_dict)
     TEST.api_ports_sdk.append(sdk_port.Port(**port_dict))
     TEST.ports.add(neutron.Port(port_dict))
 
@@ -1101,7 +1089,8 @@ def data(TEST):
     }
 
     TEST.ip_availability.add(availability)
-    TEST.api_ip_availability.add(availability)
+    TEST.api_ip_availability_sdk.append(
+        sdk_ip_availability.NetworkIPAvailability(**availability))
 
     # qos policies
     policy_dict = {'id': 'a21dcd22-7189-cccc-aa32-22adafaf16a7',
@@ -1118,8 +1107,7 @@ def data(TEST):
     # qos rule - dscp mark
     dscp_mark_rule_dict = {
         "id": "5f126d84-551a-4dcf-bb01-0e9c0df0c794",
-        "dscp_mark": 26,
-        "tenant_id": "1"
+        "dscp_mark": 26
     }
     TEST.api_dscp_mark_rule.add(dscp_mark_rule_dict)
     TEST.dscp_mark_rule.add(neutron.DSCPMarkingRule(dscp_mark_rule_dict))
@@ -1129,8 +1117,7 @@ def data(TEST):
         "id": "5f126d84-551a-4dcf-bb01-0e9c0df0c793",
         "max_kbps": 10000,
         "max_burst_kbps": 0,
-        "direction": "egress",
-        "tenant_id": "1"
+        "direction": "egress"
     }
     TEST.api_bandwidth_limit_rule.add(bandwidth_limit_rule)
     TEST.bandwidth_limit_rule.add(neutron.BandwidthLimitRule(
@@ -1140,8 +1127,7 @@ def data(TEST):
     minimum_bandwidth_rule = {
         "id": "1eddf7af-0b4c-42c5-8ae1-390b32f1de08",
         "min_kbps": 10000,
-        "direction": "egress",
-        "tenant_id": "1"
+        "direction": "egress"
     }
     TEST.api_minimum_bandwidth_rule.add(minimum_bandwidth_rule)
     TEST.minimum_bandwidth_rule.add(neutron.MinimumBandwidthRule(
@@ -1151,8 +1137,7 @@ def data(TEST):
     minimum_packet_rate_rule = {
         "id": "1eddf7af-0b4c-42c5-8ae1-390b32f1de08",
         "min_kpps": 10000,
-        "direction": "egress",
-        "tenant_id": "1"
+        "direction": "egress"
     }
     TEST.api_minimum_packet_rate_rule.add(minimum_packet_rate_rule)
     TEST.minimum_packet_rate_rule.add(neutron.MinimumPacketRateRule(
@@ -1243,7 +1228,6 @@ def data(TEST):
         'port_id': tdata['parent']['id'],
         'id': tdata['trunk_id']
     }
-    TEST.api_tp_trunks.add(tp_trunk_dict)
     TEST.api_tp_trunks_sdk.append(sdk_trunk.Trunk(**tp_trunk_dict))
 
     #    port parent
@@ -1273,7 +1257,6 @@ def data(TEST):
                            'port_id': tdata['child2']['id']}],
             'trunk_id': tdata['trunk_id']}
     }
-    TEST.api_tp_ports.add(parent_port_dict)
     TEST.api_tp_ports_sdk.append(sdk_port.Port(**parent_port_dict))
     TEST.tp_ports.add(neutron.PortTrunkParent(parent_port_dict))
 
@@ -1294,7 +1277,6 @@ def data(TEST):
         'binding:host_id': 'host',
         'security_group_ids': [tdata['security_group']]
     }
-    TEST.api_tp_ports.add(child1_port_dict)
     TEST.api_tp_ports_sdk.append(sdk_port.Port(**child1_port_dict))
     TEST.tp_ports.add(neutron.PortTrunkSubport(
         child1_port_dict,
@@ -1319,7 +1301,6 @@ def data(TEST):
         'binding:host_id': 'host',
         'security_group_ids': [tdata['security_group']]
     }
-    TEST.api_tp_ports.add(port_dict)
     TEST.api_tp_ports_sdk.append(sdk_port.Port(**port_dict))
     TEST.tp_ports.add(neutron.Port(port_dict))
 
@@ -1342,7 +1323,6 @@ def data(TEST):
         'binding:host_id': 'host',
         'security_group_ids': [tdata['security_group']]
     }
-    TEST.api_tp_ports.add(child2_port_dict)
     TEST.api_tp_ports_sdk.append(sdk_port.Port(**child2_port_dict))
     TEST.tp_ports.add(neutron.PortTrunkSubport(
         child2_port_dict,
